@@ -37,7 +37,9 @@ def get_dir() -> str:
 #######################################################################
 #     SET_DIR                                                         #
 #######################################################################
-def set_dir(directory: str) -> bool:
+def set_dir(
+    directory: str = r'./'
+) -> bool:
     if os.path.exists(directory) is False:
         return False
 
@@ -50,31 +52,56 @@ def set_dir(directory: str) -> bool:
 #######################################################################
 #     LOAD                                                            #
 #######################################################################
-def load(filename: str, force_reload: bool = False) -> bool:
+def load(
+    config: str = None,
+    force_reload: bool = False
+) -> bool:
     file_data: dict = None
 
+    if config is None:
+        return False
+
     mtomlc._mutex.acquire()
 
     for file in mtomlc._files:
-        if filename.lower() == str(file[r'filename']):
+        if config.lower() == str(file[r'config']):
             if force_reload is False:
                 mtomlc._mutex.release()
 
-                print('[{0}] Configuration \'{1}\' failed to load due to previous unsaved changes. (force_reload: False)'.format(datetime.now().strftime('%m/%d %I:%M %p'), filename), file=stderr)
+                print(
+                    '[{0}] Configuration \'{1}\' failed to load due to previous unsaved changes. (force_reload: False)'.format(
+                        datetime.now().strftime('%m/%d %I:%M %p'),
+                        config,
+                    ),
+                    file = stderr,
+                )
 
                 return False
 
             try:
-                with open(mtomlc._dir + filename + r'.toml', r'rb') as toml_file:
+                with open(mtomlc._dir + config + r'.toml', r'rb') as toml_file:
                     file_data = tomli.load(toml_file, parse_float=Decimal)
 
             except Exception as toml_exception:
-                print('[{0}] An exception was thrown while attempting to reload the configuration \'{1}\': {2}'.format(datetime.now().strftime('%m/%d %I:%M %p'), filename, str(toml_exception)), file=stderr)
+                print(
+                    '[{0}] An exception was thrown while attempting to reload the configuration \'{1}\': {2}'.format(
+                        datetime.now().strftime('%m/%d %I:%M %p'),
+                        config,
+                        str(toml_exception),
+                    ),
+                    file = stderr,
+                )
 
             if file_data is None:
                 mtomlc._mutex.release()
 
-                print('[{0}] The configuration \'{1}\' failed to load.'.format(datetime.now().strftime('%m/%d %I:%M %p'), filename), file=stderr)
+                print(
+                    '[{0}] The configuration \'{1}\' failed to load.'.format(
+                        datetime.now().strftime('%m/%d %I:%M %p'),
+                        config,
+                    ),
+                    file = stderr,
+                )
 
                 return False
 
@@ -83,45 +110,74 @@ def load(filename: str, force_reload: bool = False) -> bool:
 
             mtomlc._mutex.release()
 
-            print('[{0}] Configuration \'{1}\' loaded successfully.'.format(datetime.now().strftime('%m/%d %I:%M %p'), filename), file=stderr)
+            print(
+                '[{0}] Configuration \'{1}\' loaded successfully.'.format(
+                    datetime.now().strftime('%m/%d %I:%M %p'),
+                    config,
+                ),
+                file = stderr,
+            )
 
             return True
 
     try:
-        with open(mtomlc._dir + filename + r'.toml', r'rb') as toml_file:
+        with open(mtomlc._dir + config + r'.toml', r'rb') as toml_file:
             file_data = tomli.load(toml_file, parse_float=Decimal)
 
     except Exception as toml_exception:
-        print('[{0}] An exception was thrown while attempting to load the configuration \'{1}\': {2}'.format(datetime.now().strftime('%m/%d %I:%M %p'), filename, str(toml_exception)), file=stderr)
+        print(
+            '[{0}] An exception was thrown while attempting to load the configuration \'{1}\': {2}'.format(
+                datetime.now().strftime('%m/%d %I:%M %p'),
+                config,
+                str(toml_exception),
+            ),
+            file = stderr,
+        )
 
     if file_data is None:
         mtomlc._mutex.release()
 
-        print('[{0}] The configuration \'{1}\' failed to load.'.format(datetime.now().strftime('%m/%d %I:%M %p'), filename), file=stderr)
+        print(
+            '[{0}] The configuration \'{1}\' failed to load.'.format(
+                datetime.now().strftime('%m/%d %I:%M %p'),
+                config,
+            ),
+            file = stderr,
+        )
 
         return False
 
-    file_header = {
-        r'filename': filename.lower(),
-        r'unsaved_changes': False,
-        r'data': file_data,
-    }
+    mtoml_file = _mtoml_file(
+        config = config.lower(),
+        data = file_data
+    )
 
-    mtomlc._files.append(file_header)
+    mtomlc._files.append(mtoml_file)
     mtomlc._mutex.release()
 
-    print('[{0}] Configuration \'{1}\' loaded successfully.'.format(datetime.now().strftime('%m/%d %I:%M %p'), filename), file=stderr)
+    print(
+        '[{0}] Configuration \'{1}\' loaded successfully.'.format(
+            datetime.now().strftime('%m/%d %I:%M %p'),
+            config,
+        ),
+        file = stderr,
+    )
 
     return True
 
 #######################################################################
 #     IS_LOADED                                                       #
 #######################################################################
-def is_loaded(filename: str) -> bool:
+def is_loaded(
+    config: str = None
+) -> bool:
+    if config is None:
+        return False
+
     mtomlc._mutex.acquire()
 
     for file in mtomlc._files:
-        if filename.lower() == str(file[r'filename']):
+        if config.lower() == str(file[r'config']):
             mtomlc._mutex.release()
 
             return True
@@ -133,13 +189,19 @@ def is_loaded(filename: str) -> bool:
 #######################################################################
 #     SAVE                                                            #
 #######################################################################
-def save(filename: str, force_overwrite: bool = False) -> bool:
+def save(
+    config: str = None,
+    force_overwrite: bool = False
+) -> bool:
     is_valid_toml: bool = True
 
+    if config is None:
+        return False
+
     mtomlc._mutex.acquire()
 
     for file in mtomlc._files:
-        if filename.lower() == str(file[r'filename']):
+        if config.lower() == str(file[r'config']):
             if force_overwrite is False and file[r'unsaved_changes'] is False:
                 mtomlc._mutex.release()
 
@@ -151,7 +213,14 @@ def save(filename: str, force_overwrite: bool = False) -> bool:
             except Exception as toml_exception:
                 is_valid_toml = False
 
-                print('[{0}] An exception was thrown while preparing to save the configuration \'{1}\': {2}'.format(datetime.now().strftime('%m/%d %I:%M %p'), filename, str(toml_exception)), file=stderr)
+                print(
+                    '[{0}] An exception was thrown while preparing to save the configuration \'{1}\': {2}'.format(
+                        datetime.now().strftime('%m/%d %I:%M %p'),
+                        config,
+                        str(toml_exception),
+                    ),
+                    file = stderr,
+                )
 
             if is_valid_toml is False:
                 mtomlc._mutex.release()
@@ -159,7 +228,7 @@ def save(filename: str, force_overwrite: bool = False) -> bool:
                 return False
 
             try:
-                with open(mtomlc._dir + filename + r'.toml', r'wb') as toml_file:
+                with open(mtomlc._dir + config + r'.toml', r'wb') as toml_file:
                     tomli_w.dump(file[r'data'], toml_file)
 
                 file[r'unsaved_changes'] = False
@@ -167,7 +236,14 @@ def save(filename: str, force_overwrite: bool = False) -> bool:
             except Exception as toml_exception:
                 is_valid_toml = False
 
-                print('[{0}] An exception was thrown while saving the configuration \'{1}\': {2}'.format(datetime.now().strftime('%m/%d %I:%M %p'), filename, str(toml_exception)), file=stderr)
+                print(
+                    '[{0}] An exception was thrown while saving the configuration \'{1}\': {2}'.format(
+                        datetime.now().strftime('%m/%d %I:%M %p'),
+                        config,
+                        str(toml_exception),
+                    ),
+                    file = stderr,
+                )
 
             mtomlc._mutex.release()
 
@@ -175,14 +251,22 @@ def save(filename: str, force_overwrite: bool = False) -> bool:
 
     mtomlc._mutex.release()
 
-    print('[{0}] The configuration \'{1}\' does not exist.'.format(datetime.now().strftime('%m/%d %I:%M %p'), filename))
+    print(
+        '[{0}] The configuration \'{1}\' does not exist.'.format(
+            datetime.now().strftime('%m/%d %I:%M %p'),
+            config,
+        ),
+        file = stderr,
+    )
 
     return False
 
 #######################################################################
 #     SAVE_ALL                                                        #
 #######################################################################
-def save_all(force_overwrite: bool = False) -> bool:
+def save_all(
+    force_overwrite: bool = False
+) -> bool:
     saved_all: bool = True
 
     mtomlc._mutex.acquire()
@@ -202,13 +286,20 @@ def save_all(force_overwrite: bool = False) -> bool:
             is_valid_toml = False
             saved_all = False
 
-            print('[{0}] An exception was thrown while preparing to save the configuration \'{1}\': {2}'.format(datetime.now().strftime('%m/%d %I:%M %p'), str(file[r'filename']), str(toml_exception)), file=stderr)
+            print(
+                '[{0}] An exception was thrown while preparing to save the configuration \'{1}\': {2}'.format(
+                    datetime.now().strftime('%m/%d %I:%M %p'),
+                    str(file[r'config']),
+                    str(toml_exception)
+                ),
+                file = stderr
+            )
 
         if is_valid_toml is False:
             continue
 
         try:
-            with open(mtomlc._dir + str(file[r'filename']) + r'.toml', r'wb') as toml_file:
+            with open(mtomlc._dir + str(file[r'config']) + r'.toml', r'wb') as toml_file:
                 tomli_w.dump(file[r'data'], toml_file)
 
             file[r'unsaved_changes'] = False
@@ -216,7 +307,14 @@ def save_all(force_overwrite: bool = False) -> bool:
         except Exception as toml_exception:
             saved_all = False
 
-            print('[{0}] An exception was thrown while saving the configuration \'{1}\': {2}'.format(datetime.now().strftime('%m/%d %I:%M %p'), str(file[r'filename']), str(toml_exception)), file=stderr)
+            print(
+                '[{0}] An exception was thrown while saving the configuration \'{1}\': {2}'.format(
+                    datetime.now().strftime('%m/%d %I:%M %p'),
+                    str(file[r'config']),
+                    str(toml_exception),
+                ),
+                file = stderr,
+            )
 
     mtomlc._mutex.release()
 
@@ -225,17 +323,35 @@ def save_all(force_overwrite: bool = False) -> bool:
 #######################################################################
 #     GET                                                             #
 #######################################################################
-def get(filename: str, field: str) -> Any:
+def get(
+    config: str = None,
+    group: str = None,
+    field: str = None
+) -> Any:
+    if config is None or field is None:
+        return None
+
     mtomlc._mutex.acquire()
 
     for file in mtomlc._files:
-        if filename.lower() == str(file[r'filename']):
-            if field in file[r'data'].keys():
-                value = file[r'data'][field]
+        if config.lower() == str(file[r'config']):
+            if group is None:
+                if field in file[r'data'].keys():
+                    value = file[r'data'][field]
 
-                mtomlc._mutex.release()
+                    mtomlc._mutex.release()
 
-                return value
+                    return value
+
+            else:
+                if group in file[r'data'].keys():
+                    if type(file[r'data'][group]) is dict:
+                        if field in file[r'data'][group].keys():
+                            value = file[r'data'][group][field]
+
+                            mtomlc._mutex.release()
+
+                            return value
 
     mtomlc._mutex.release()
 
@@ -244,16 +360,32 @@ def get(filename: str, field: str) -> Any:
 #######################################################################
 #     SET                                                             #
 #######################################################################
-def set(filename: str, field: str, value: Any) -> bool:
+def set(
+    config: str = None,
+    group: str = None,
+    field: str = None,
+    value: Any = None
+) -> bool:
     is_valid_toml: bool = True
 
+    if config is None or field is None:
+        return False
+
     mtomlc._mutex.acquire()
 
     for file in mtomlc._files:
-        if filename.lower() == str(file[r'filename']):
+        if config.lower() == str(file[r'config']):
             try:
                 toml_copy = copy.deepcopy(file[r'data'])
-                toml_copy[field] = value
+
+                if group is None:
+                    toml_copy[field] = value
+
+                else:
+                    if group not in toml_copy.keys():
+                        toml_copy[group] = {}
+
+                    toml_copy[group][field] = value
 
                 # this will throw an exception if the modifications resulted in an invalid toml format
                 tomli.loads(tomli_w.dumps(toml_copy))
@@ -264,7 +396,28 @@ def set(filename: str, field: str, value: Any) -> bool:
             except Exception as toml_exception:
                 is_valid_toml = False
 
-                print('[{0}] The configuration \'{1}\' and field \'{2}\' attempted value set resulted in an invalid TOML format: {3}'.format(datetime.now().strftime('%m/%d %I:%M %p'), filename, field, str(toml_exception)), file=stderr)
+                if group is None:
+                    print(
+                        '[{0}] The configuration \'{1}\' and field \'{2}\' attempted value set resulted in an invalid TOML format: {3}'.format(
+                            datetime.now().strftime('%m/%d %I:%M %p'),
+                            config,
+                            field,
+                            str(toml_exception),
+                        ),
+                        file = stderr,
+                    )
+
+                else:
+                    print(
+                        '[{0}] The configuration \'{1}\', group \'{3}\', and field \'{2}\' attempted group value set resulted in an invalid TOML format: {4}'.format(
+                            datetime.now().strftime('%m/%d %I:%M %p'),
+                            config,
+                            field,
+                            group,
+                            str(toml_exception),
+                        ),
+                        file = stderr,
+                    )
 
             mtomlc._mutex.release()
 
@@ -272,10 +425,28 @@ def set(filename: str, field: str, value: Any) -> bool:
 
     mtomlc._mutex.release()
 
-    print('[{0}] The configuration \'{1}\' does not exist.'.format(datetime.now().strftime('%m/%d %I:%M %p'), filename), file=stderr)
+    print(
+        '[{0}] The configuration \'{1}\' does not exist.'.format(
+            datetime.now().strftime('%m/%d %I:%M %p'),
+            config,
+        ),
+        file = stderr,
+    )
 
     return False
 
+#######################################################################
+#         _MTOML_FILE                                                 #
+#######################################################################
+class _mtoml_file(dict):
+    ###################################################################
+    #     CONSTRUCTOR, INSTANCE VARIABLES                             #
+    ###################################################################
+    def __init__(self: r'_mtoml_file', config: str = None, data: dict = None):
+        self[r'config'] = config
+        self[r'unsaved_changes'] = False
+        self[r'data'] = data
+
 #######################################################################
 #                                                                     #
 #         MTOMLC                                                      #
@@ -285,61 +456,103 @@ class mtomlc:
     ###################################################################
     #     CLASS VARIABLES                                             #
     ###################################################################
-    _files: list = []  # { 'filename': str, 'unsaved_changes': bool, 'data': dict }
+    _files: list[_mtoml_file] = []
     _dir: str = r'./'
     _mutex: Type[Lock] = Lock()
 
     ###################################################################
     #     CONSTRUCTOR, INSTANCE VARIABLES                             #
     ###################################################################
-    def __init__(self: r'mtomlc', directory: str = r'./') -> None:
+    def __init__(
+        self: r'mtomlc',
+        directory: str = r'./'
+    ) -> None:
         if r'./' != directory:
             set_dir(directory)
 
     ###################################################################
     #     GET_DIR                                                     #
     ###################################################################
-    def get_dir(self: r'mtomlc') -> str:
+    def get_dir(
+        self: r'mtomlc'
+    ) -> str:
         return get_dir()
 
     ###################################################################
     #     SET_DIR                                                     #
     ###################################################################
-    def set_dir(self: r'mtomlc', directory: str) -> bool:
+    def set_dir(
+        self: r'mtomlc',
+        directory: str = r'./',
+    ) -> bool:
         return set_dir(directory)
 
     ###################################################################
     #     LOAD                                                        #
     ###################################################################
-    def load(self: r'mtomlc', filename: str, force_reload: bool = False) -> bool:
-        return load(filename, force_reload)
+    def load(
+        self: r'mtomlc',
+        config: str = None,
+        force_reload: bool = False,
+    ) -> bool:
+        return load(config, force_reload)
 
     ###################################################################
     #     IS_LOADED                                                   #
     ###################################################################
-    def is_loaded(self: r'mtomlc', filename: str) -> bool:
-        return is_loaded(filename)
+    def is_loaded(
+        self: r'mtomlc',
+        config: str = None,
+    ) -> bool:
+        return is_loaded(config)
 
     ###################################################################
     #     SAVE                                                        #
     ###################################################################
-    def save(self: r'mtomlc', filename: str, force_overwrite: bool = False) -> bool:
-        return save(filename, force_overwrite)
+    def save(
+        self: r'mtomlc',
+        config: str = None,
+        force_overwrite: bool = False,
+    ) -> bool:
+        return save(config, force_overwrite)
 
     ###################################################################
     #     SAVE_ALL                                                    #
     ###################################################################
-    def save_all(self: r'mtomlc', force_overwrite: bool = False) -> bool:
+    def save_all(
+        self: r'mtomlc',
+        force_overwrite: bool = False,
+    ) -> bool:
         return save_all(force_overwrite)
 
     ###################################################################
     #     GET                                                         #
     ###################################################################
-    def get(self: r'mtomlc', filename: str, field: str) -> Any:
-        return get(filename, field)
+    def get(
+        self: r'mtomlc',
+        config: str = None,
+        group: str = None,
+        field: str = None,
+    ) -> Any:
+        return get(
+            config = config,
+            group = group,
+            field = field,
+        )
 
     ###################################################################
     #     SET                                                         #
     ###################################################################
-    def set(self: r'mtomlc', filename: str, field: str, value: Any) -> bool:
-        return set(filename, field, value)
+    def set(
+        self: r'mtomlc',
+        config: str = None,
+        group: str = None,
+        field: str = None,
+        value: Any = None,
+    ) -> bool:
+        return set(
+            config = config,
+            group = group,
+            field = field,
+            value = value,
+        )
